#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <err.h>

#include "pcifunc.h"

#define LONGRUN_VID	0x1279
#define LONGRUN_DID	0x0395
#define LONGRUN_FAST	0x0E
#define LONGRUN_SLOW	0x11
#define LONGRUN_DEFAULT	0x01
#define LONGRUN_REG	0xA8

void usage(progname)
	char *progname;
{
	fprintf(stderr, "usage:	%s -d		(restore longrun default)\n",
	    progname);
	fprintf(stderr, "	%s -f		(set longrun to fast)\n",
	    progname);
	fprintf(stderr, "	%s -s		(set longrun to slow)\n",
	    progname);
	fprintf(stderr, "	%s -S speed	(set longrun to 'speed')\n",
	    progname);

	exit(1);
}

int main(argc, argv)
	int argc;
	char *argv[];
{
	int c, fd, speed;

	if ((fd = pci_find_device(LONGRUN_VID, LONGRUN_DID)) == -1)
		errx(1, "couldn't find longrun device");

	while ((c = getopt(argc, argv, "dfsS:")) != -1) {
		switch (c) {
		case 'd':
			pci_config_write_u8(fd, LONGRUN_REG,
			    LONGRUN_DEFAULT);
			warnx("set longrun speed to power-up default");
			exit(0);
			break;
		case 's':
			pci_config_write_u8(fd, LONGRUN_REG,
			    LONGRUN_SLOW);
			warnx("set longrun speed to slow");
			exit(0);
			break;
		case 'f':
			pci_config_write_u8(fd, LONGRUN_REG,
			    LONGRUN_FAST);
			warnx("set longrun speed to fast");
			exit(0);
			break;
		case 'S':
			speed = atol(optarg);
			if (speed < 0 || speed > 7) {
				warnx("invalid CPU speed: %d", speed);
				errx(1, "only values 0 and 7 are allowed");
			}
			pci_config_write_u8(fd, LONGRUN_REG,
			    LONGRUN_SLOW|speed << 1);
			warnx("set longrun speed to %d", speed);
			exit(0);
			break;
		default:
			usage(argv[0]);
			exit(1);
			break;
		}

	}

	usage(argv[0]);
	exit(1);
}
