/* manipulate PCI devices from user space

   Tridge, July 2000
*/
/* 
   Copyright (C) Andrew Tridgell 2000
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "capture.h"
#include <err.h>

#define	_PATH_DEVPCI	"/dev/pci"
static int devidx = 0;
struct pcisel devs[10];
static int fd = -1;

static void fdinit()
{
	fd = open(_PATH_DEVPCI, O_RDWR, 0);
	if (fd < 0)
		err(1, "%s", _PATH_DEVPCI);
}

/* find a PCI device and return a handle to it */
int pci_find_device(u32 vendor, u32 device)
{
	struct pci_match_conf match;
	struct pci_conf pdevs[10];
	struct pci_conf_io pc;
	int idx;

	if (fd == -1)
		fdinit();

	bzero((char *)&match, sizeof(match));
	bzero((char *)&devs, sizeof(devs));
	bzero((char *)&pc, sizeof(pc));
	match.pc_vendor = vendor;
	match.pc_device = device;
	match.flags = PCI_GETCONF_MATCH_VENDOR|PCI_GETCONF_MATCH_DEVICE;

	pc.pat_buf_len = sizeof(match);
	pc.num_patterns = 1;
	pc.patterns = &match;
	pc.match_buf_len = sizeof(pdevs);
	pc.matches = pdevs;

	if (ioctl(fd, PCIOCGETCONF, &pc) == -1)
		err(1, "ioctl(PCIOCGETCONF)");

	if (pc.num_matches != 1)
		return(-1);

	idx = devidx;
	bcopy((char *)&pdevs[0].pc_sel, (char *)&devs[devidx],
		sizeof(struct pcisel));
	devidx++;

	return(idx);
}

static int readit(struct pcisel sel, unsigned long reg, int width)
{
	struct pci_io pi;

	pi.pi_sel = sel;
	pi.pi_reg = reg; /* XXX error check */
	pi.pi_width = width;

	if (ioctl(fd, PCIOCREAD, &pi) < 0)
		err(1, "ioctl(PCIOCREAD)");

	return(pi.pi_data);
}

static int writeit(struct pcisel sel, unsigned long reg, unsigned long data, int width)
{
	struct pci_io pi;

	pi.pi_sel = sel;
	pi.pi_reg = reg;
	pi.pi_width = width;
	pi.pi_data = data;

	if (ioctl(fd, PCIOCWRITE, &pi) < 0)
		err(1, "ioctl(PCIOCWRITE)");

	return(0);
}

/* routines to read and write PCI config space */
int pci_config_write_u8(int fd, int ofs, u8 v)
{
	return (writeit(devs[fd], ofs, v, 1));
}

int pci_config_write_u16(int fd, int ofs, u16 v)
{
	return (writeit(devs[fd], ofs, v, 2));
}

int pci_config_write_u32(int fd, int ofs, u32 v)
{
	return (writeit(devs[fd], ofs, v, 4));
}

int pci_config_read_u8(int fd, int ofs, u8 *v)
{
	*v = readit(devs[fd], ofs, 1);
	return (0);
}

int pci_config_read_u16(int fd, int ofs, u16 *v)
{
	*v = readit(devs[fd], ofs, 2);
	return (0);
}

int pci_config_read_u32(int fd, int ofs, u32 *v)
{
	*v = readit(devs[fd], ofs, 4);
	return (0);
}

/* find a pci base address via /proc/bus/pci/devices. This seems to be
   needed on some boxes. Why? */
u32 pci_read_base_address(u32 vendor, u32 device)
{
	return 0;
}
